#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <gtk/gtk.h>

#include "main.h"

static gboolean delete_event(GtkWidget *widget,
			     GdkEvent *event,
			     gpointer data)
{
  g_print("delete event occured\n");

  gtk_main_quit();

  return FALSE;
}


static void destroy(GtkWidget *widget,
		    gpointer data)
{
  g_print("destroy signal occured\n");

  gtk_main_quit();
}

static void download_signal(GtkWidget *widget,
			      gpointer data)
{
    g_print("download_callback: %s\n", (char *) data);

    if (strcmp((char *) data, "close") == 0) {
      gtk_main_quit();
    }
}

static gint download(char *url)

{
  int pipefd[2];
  pid_t pid;

  pipe(pipefd); /* FIXME: check return value */

  if ((pid = fork()) == 0) { /* child */
    close(pipefd[0]);
    dup2(pipefd[1], STDERR_FILENO); /* FIXME: check return value */
    execlp(WGET, WGET, url, NULL); /* FIXME: check return value */
  } else if (pid == -1) { /* error */
    /* FIXME: add gracious fail */
  }
  
  /* parent */

  close(pipefd[1]);

  return pipefd[0];
}

static void download_input(gpointer data,
			   gint source,
			   GdkInputCondition condition)
{
  char buf[128];
  int bufsiz = 128; /* FIXME: hackish */
  int n;
  int status;
  char *str;

  struct download_details *details = data;

  if ((n = read(source, buf, bufsiz)) > 0) { /* ok */
    gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(details->buffer), buf, n);
  }
  else if (n == 0) { /* eof */
    gdk_input_remove(details->tag);
    wait(&status); /* NOTE: waitpid() on an explicit pid would be preferable */
    if (WIFEXITED(status)) {
      asprintf(&str, "normal termination, exit status = %d",
	       WEXITSTATUS(status));
      gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(details->buffer),
				       str, -1);
      free(str);
    }
    else {
      gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(details->buffer),
				       "abnormal termination", -1);
    }
  }
  else { /* n == -1, error */
    /* FIXME: add error handling */
  }
}

int main(int argc, char **argv)
{
  GtkWidget *window;
  GtkWidget *table;

  GtkWidget *view;
  GtkTextBuffer *buffer;

  gint fd;
  struct download_details details;

  GtkWidget *button;

  gtk_init(&argc, &argv);
    
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "DHT for GNUnet");

  g_signal_connect(G_OBJECT(window), "delete_event",
		   G_CALLBACK(delete_event), NULL);

  g_signal_connect(G_OBJECT(window), "destroy",
		   G_CALLBACK(destroy), NULL);

  gtk_container_set_border_width(GTK_CONTAINER(window), 10);

  /* table */

  table = gtk_table_new(2, 1, TRUE);

  /* text view */

  view = gtk_text_view_new();
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
  gtk_widget_show(view);
  gtk_table_attach_defaults(GTK_TABLE(table), view, 0, 1, 0, 1);

  /* button */

  button = gtk_button_new_with_label("Close");
  g_signal_connect(G_OBJECT(button), "clicked",
		   G_CALLBACK (download_signal), (gpointer) "close");
  gtk_widget_show(button);
  gtk_table_attach_defaults(GTK_TABLE(table), button, 0, 1, 1, 2);


  /* container */

  gtk_container_add(GTK_CONTAINER(window), table);
  
  gtk_widget_show(table);
  gtk_widget_show(window);

  /* timeout */

  fd = download(URL);
  
  details.buffer = buffer;
  details.tag = gdk_input_add(fd, GDK_INPUT_READ,
			      download_input, &details);
  
  /*
  g_timeout_add(1000, timeout_callback, buffer);
  */

  /* main */

  gtk_main();
    
  return 0;
}
