/**
 * 
 */

/**
 * @file include/api_structs.h
 * @brief Definitions for the structs that are used in DHT's
 *        application programming interface.
 * @author Tomi Tukiainen
 * @version $Id: api_structs.h,v 1.10 2004/05/11 11:22:36 tmtoivon Exp $ 
 */

#ifndef DHT_API_STRUCTS_H
#define DHT_API_STRUCTS_H

#include "gnunet_core.h"

typedef HashCode160 DHT_TableId; 
typedef HostIdentity DHT_NodeAddress; 

typedef struct DHT_TableHandle_t {
  unsigned int errorCode;
  DHT_TableId tableId; 
} DHT_TableHandle; 

typedef struct DHT_TableMetaData_t {
  char *name; 
} DHT_TableMetaData; 

typedef struct DHT_TableConfig_t {
  unsigned int replicationCount;
  unsigned int parallelismCount;
  unsigned int maximumValuesPerKey;
  unsigned int expirationTimeSeconds;
  unsigned int flags;
  float cacheTimeMultiplier;
} DHT_TableConfig; 

typedef struct DHT_DataContainer_t {
  unsigned int dataLength;
  void *data;
} DHT_DataContainer;

typedef struct DHT_ResultSetItem_t {
  DHT_DataContainer * data;
  struct DHT_ResultSetItem_t *nextItem;
} DHT_ResultSetItem;

typedef struct DHT_ResultSet_t {
  unsigned int errorCode;
  DHT_ResultSetItem *firstItem;
} DHT_ResultSet;

typedef struct DHT_TableSetItem_t {
  DHT_TableId tableId;
  DHT_TableMetaData tableMetaData;
  struct DHT_TableSetItem_t *nextItem;
} DHT_TableSetItem; 

typedef struct DHT_TableSet_t {
  unsigned int errorCode;
  DHT_TableSetItem *firstItem;
} DHT_TableSet;

typedef struct DHT_StoredDataReference_t {
  HashCode160 hashCode;
  cron_t insertionTime; 
} DHT_StoredDataReference; 

typedef struct DHT_DataListItem_t {
  DHT_DataContainer *key;
  DHT_DataContainer *value;
  DHT_StoredDataReference uniqueReference;
  struct DHT_DataListItem_t *nextItem;
} DHT_DataListItem;

typedef struct DHT_DataList_t {
  unsigned int errorCode;
  DHT_DataListItem *firstItem;
} DHT_DataList;

typedef struct DHT_NodeSetItem_t {
  DHT_NodeAddress node;
  struct DHT_NodeSetItem_t *nextItem;
} DHT_NodeSetItem; 

typedef struct DHT_NodeSet_t {
  unsigned int errorCode;
  DHT_NodeSetItem *firstItem;
} DHT_NodeSet;

#endif /* DHT_API_STRUCTS_H */
