/*
      This file is part of GNUnet

      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
      by the Free Software Foundation; either version 2, or (at your
      option) any later version.

      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
      General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
      Free Software Foundation, Inc., 59 Temple Place - Suite 330,
      Boston, MA 02111-1307, USA.
 */
 
/**
 * This file describes the interface for the persistance layer of a DHT module.
 * 
 * @file module/datalayer.h
 * @brief Interface and datastructures for persistance of DHT's localstorage
 * @author Simo Viitanen
 * @version $Id: datalayer_persistance.h,v 1.4 2004/05/11 09:32:43 szviitan Exp $ 
 */


#include "gnunet_core.h"    /* for HashCode160               */
#include "api_structs.h"    /* for dht's structs             */
#include "datalayer.h"

#ifndef DATALAYER_PERSISTANCE
#define DATALAYER_PERSISTANCE 0

/**
 * creates a table for the specified id
 * could be one file for all or one file per table
 * 
 * @param const DHT_TableId * table_id
 * @param const DHT_TableMetaDatan metadata
 * @param const DHT_TableConfig config
 *
 * @return int containing the successing information
 **/
int persistentCreateTable(
      const DHT_TableId * id, 
      const DHT_TableMetaData * metadata, 
      const DHT_TableConfig * config);

/**
 * stores one unit af data to the file
 * 
 * @param const DHT_TableId * table_id
 * @param DHT_LocalStoreUnit * unitToStore
 *
 * @return int containing the successing information
 **/
int persistentStore(
      const DHT_TableId * id, 
      DHT_LocalStoreUnit * unitToStore);

/**
 * removes one unit of data from a file
 * 
 * @param const DHT_TableId * table_id
 * @param DHT_LocalStoreUnit * unitToRemove
 *
 * @return int containing the successing information
 **/
int persistentRemove(
      const DHT_TableId * id, 
      DHT_LocalStoreUnit * unitToRemove);

/**
 * lists all units from a file. list is padded with a NULL pointer 
 * indicating the end of structs.
 * This function is called when the datastore is initialised
 * 
 * @param const DHT_TableId * id of the table
 *
 * @return DHT_LocalStoreUnit ** a pointer to the beginning of the result array
 **/
DHT_LocalStoreUnit ** persistentListData(
      const DHT_TableId * id);

/**
 * lists all files in store. list is padded with a NULL pointer 
 * indicating the end of structs.
 * This function is called when the datastore is initialised
 * 
 * @return DHT_TableDef ** a pointer to the beginning of the result array
 **/
DHT_TableDef ** persistentListTables();

/**
 * deletes a table for the specified id
 * could be one file for all or one file per table
 * 
 * @return int containing the successing information
 **/
int persistentDeleteTable(
      const DHT_TableId * id);


#endif
