#include <stdio.h>
#include "dht_api.h"

/**
 * Parse the options, set the timeout.
 * @param argc the number of options
 * @param argv the option list (including keywords)
 * @return OK on error, SYSERR if we should exit 
 **/
static int parseOptions(int argc,
			char ** argv) {
  int option_index;
  int c;  
  
  FREENONNULL(setConfigurationString("GNUNETD",
				     "LOGFILE",
				     NULL));
  while (1) {
    static struct GNoption long_options[] = {
      LONG_DEFAULT_OPTIONS,
      { "nickname", 1, 0, 'n' },
      { 0,0,0,0 }
    };    
    option_index=0;
    c = GNgetopt_long(argc,
		      argv, 
		      "vhdc:L:H:n:", 
		      long_options, 
		      &option_index);    
    if (c == -1) 
      break;  /* No more flags to process */
    if (YES == parseDefaultOptions(c, GNoptarg))
      continue;
    switch(c) {
    case 'h': 
      printf("USAGE: api_test\n");
      printHelpDefault();
      return SYSERR;
    default: 
      LOG(LOG_FAILURE,
	  "FAILURE: Unknown option %c. Aborting.\n"\
	  "Use --help to get a list of options.\n",
	  c);
      return -1;
    } /* end of parsing commandline */
  } /* while (1) */
  return OK;
}


/* Doesn't do anything VERY interesting yet. */
int main(int argc, char * argv[]) {
  int errorCode; 

  printf("api_test::main() - Initializing DHT api!\n");

  if (SYSERR == initUtil(argc, argv, &parseOptions))
    return 0; /* parse error, --help, etc. */ 

  errorCode = initializeApi(); 
  printf("api_test::main() - Initialize returned errorcode: '%d'.",errorCode); 
  
  printf("api_test::main() - Destroying DHT api!\n");
  errorCode = destroyApi();
  printf("api_test::main() - Destroying returned errorcode: '%d'.",errorCode); 

  return 0;
}
