/*
      This file is part of GNUnet

      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
      by the Free Software Foundation; either version 2, or (at your
      option) any later version.

      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
      General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
      Free Software Foundation, Inc., 59 Temple Place - Suite 330,
      Boston, MA 02111-1307, USA.
 */
 
/**
 * @file module/test/client-server_test.c
 * @brief CuTests for client-server.c
 * @author Marko Rih
 * @version $Id: client-server_test.c,v 1.6 2004/05/15 09:06:14 szviitan Exp $
 */

#include <stdio.h>
#include "CuTest.h"
#include "gnunet_util.h"
#include "gnunet_core.h"
#include "client-server.h"
#include "../../include/api_structs.h"
#include "../../include/api_errorcodes.h"
#include "../../../../server/core.h"

#include "../dht.h"
#include "../dht_util.h"

#define CS_TEST 1

#ifndef DEBUG
#define DEBUG 1
#endif

/**
 * Global core API.
 **/
CoreAPIForApplication hackz_coreAPI;

static int foobaarCS(const unsigned short type, CSHandler callback) {
  //printf("foobaarCS %d \n",type);
  return OK;
}
static int unfoobaarCS(ClientExitHandler callback)  {
  //printf("unfoobaarCS\n");
  return OK;
}
static int isfoobaar(const unsigned short type) {
  //printf("isfoobaar %d \n",type);
  return YES;
}

static int foobaarP2P(const unsigned short type, MessagePartHandler callback) {
  //printf("foobaarP2P %d \n",type);
  return OK;
}

static DHT_NodeAddress * get_NodeAddress(){
  DHT_NodeAddress * res;
  res = MALLOC(sizeof(*res));
  return res;
}

/* utility functions for */
int freeResultSet(DHT_ResultSet ** result) {
  DHT_ResultSetItem * item, * prevItem;
  item=prevItem=NULL;
  if (*result != NULL) {
    item = (*result)->firstItem;
    while(item != NULL) {
      prevItem = item;
      item = prevItem->nextItem;
      FREE(prevItem->data);
      FREE(prevItem);
    }
    FREE(*result);
    *result = NULL;
  }
  return OK;
}

int freeDataList(DHT_DataList **result) {
  DHT_DataListItem  * item, 
                    * prevItem;
  item=prevItem=NULL;
  if (*result != NULL) {
    item = (*result)->firstItem;
    while(item != NULL) {
      prevItem = item;
      item = prevItem->nextItem;
      FREE(prevItem->key);
      FREE(prevItem->value);
      FREE(prevItem);
    }
    FREE(*result);
    *result = NULL;
  }
  return OK;
}

int freeTableSet(DHT_TableSet ** result) {
  DHT_TableSetItem  * item, 
                    * prevItem;
  item=prevItem=NULL;
  if (*result != NULL) {
    item = (*result)->firstItem;
    while(item != NULL) {
      prevItem = item;
      item = prevItem->nextItem;
      FREE(prevItem);
    }
    FREE(*result);
    *result = NULL;
  }
  return OK;
}

/* Dummy functions for testing instead of really using dht.c */

int create(DHT_TableConfig * tableConfig,
           DHT_TableMetaData * metaData,
           DHT_TableId ** tableId) {
  return OK;
}

int join(DHT_NodeAddress * nodeAddress,
         DHT_TableId * tableId) {
  return OK;
}

int leave(DHT_TableId * tableId) {
  return OK;
}

int insert(DHT_TableId * tableId,
           DHT_DataContainer * key,
           DHT_DataContainer * value) {
  return OK;
}

int fetch(DHT_TableId * tableId,
          DHT_DataContainer * key,
          DHT_ResultSet ** result) {
  return OK;
}

int tables(DHT_NodeAddress * nodeAddress,
           DHT_TableSet ** tableSet) {
  return OK;
}

int inserted(DHT_TableId * tableId,
             DHT_DataList ** result) {
  return OK;
}

int drop(DHT_TableId * tableId,
         DHT_StoredDataReference * dataReference) {
  return OK;
}
void test_clientServerRequestApiId(CuTest *tc) {

}

CuSuite* clientServer_GetSuite() {
  printf("getSuite\n");
  CuSuite* suite = CuSuiteNew();
  SUITE_ADD_TEST(suite, test_clientServerRequestApiId);
  //SUITE_ADD_TEST(suite, test_client-serverRequestCreate);
  //SUITE_ADD_TEST(suite, test_client-serverRequestiJoin);
  //SUITE_ADD_TEST(suite, test_client-serverRequestLeave);
  //SUITE_ADD_TEST(suite, test_client-serverRequestInsert);
  //SUITE_ADD_TEST(suite, test_client-serverRequestFetch);
  //SUITE_ADD_TEST(suite, test_client-serverRequestTables);
  //SUITE_ADD_TEST(suite, test_client-serverRequestInserted);
  //SUITE_ADD_TEST(suite, test_client-serverRequestDrop);
  //SUITE_ADD_TEST(suite, test_client-serverRequestStatus);
  return suite;
}

#ifndef TEST_ALL
int main(void) {
  int errorCode = OK;

  printf("start testing CS\n");
  hackz_coreAPI.myIdentity = get_NodeAddress();
  hackz_coreAPI.registerHandler = &foobaarP2P;
  hackz_coreAPI.isHandlerRegistered = &isfoobaar;
  hackz_coreAPI.unregisterHandler = &foobaarP2P;
  hackz_coreAPI.registerClientHandler = &foobaarCS;
  hackz_coreAPI.unregisterClientHandler = &foobaarCS;
  hackz_coreAPI.registerClientExitHandler = &unfoobaarCS;
  hackz_coreAPI.unregisterClientExitHandler = &unfoobaarCS;
  hackz_coreAPI.isClientHandlerRegistered = &isfoobaar;

  CuString *output = CuStringNew();
  CuSuite* suite = CuSuiteNew();

  //initCore();
  //coreAPI = getCoreAPIForApplication();

  printf("Initializing ClientServer!\n");
  errorCode = initClientServer(&hackz_coreAPI);
  printf("Initialize returned errorcode: '%d'.",errorCode);

  CuSuiteAddSuite(suite, clientServer_GetSuite());
    
  printf("Destroying ClientServer!\n");
  errorCode = doneClientServer(&hackz_coreAPI);
  printf("Destroying returned errorcode: '%d'.",errorCode);
  //doneCore();

  CuSuiteRun(suite);
  CuSuiteSummary(suite, output);
  CuSuiteDetails(suite, output);
  printf("%s\n", output->buffer);
  return 0; 
}
#endif
