#include <gnunet_core.h>
#include <gnunet_util.h>
#include "../datalayer.h"
#include "../../include/api_structs.h"
#include "../../include/api_errorcodes.h"
#include "CuTest.h"

#define TRUE 1
#define FALSE 0

#ifndef DEBUG
#define DEBUG 1
#endif


static HashCode160 test_hash1 = {1,2,3,4, 5};

static DHT_LocalStoreUnit * get_DataUnitFromStr(char * data)
{

  HashCode160 * hashi;
  DHT_DataContainer * test_data;
  DHT_LocalStoreUnit * unit;
  
  hashi = MALLOC(sizeof(HashCode160));
  hash(data, strlen(data), hashi);

  test_data = MALLOC(sizeof(DHT_DataContainer));
  test_data->data = data;
  test_data->dataLength = strlen(data);
  
#if DEBUG
  printf("new_DHT_LocalStoreUnit %s\n",data);
#endif
  unit = new_DHT_LocalStoreUnit(hashi, test_data);
  
#if DEBUG
  printf("get_DataUnitFromStr %s\n",data);
#endif
  return unit;
}

static DHT_TableConfig * get_TableConfig(){
  DHT_TableConfig * res;

  res = MALLOC(sizeof(*res));
  res->replicationCount =0;
  res->parallelismCount=0;
  res->maximumValuesPerKey=0;
  res->expirationTimeSeconds=10;
  res->flags=0;
  res->cacheTimeMultiplier=0.0;
  return res;
}


static DHT_TableMetaData * get_TableMeta(char * tableName){
  DHT_TableMetaData * res;

  res = MALLOC(sizeof(*res));
  res->name = tableName;

  return res;
}


void test_datalayerTableCreation(CuTest *tc)
{
  DHT_LocalTableSet set;
  DHT_TableDef * def1;
  char * tableName = "TAULU";
#if DEBUG
  printf("test_datalayerTableCreation.\ninit store \n");
#endif
  initLocalStore(NULL);
#if DEBUG
  printf("create a table \n");
#endif
  if(localStoreCreateTable(&test_hash1, get_TableMeta(tableName), get_TableConfig()) == OK) {
    /* test if any tables are found */
    set =  localStoreListTables();
    CuAssertIntEquals(tc, set.returnCode,OK);
    /* test iterator */
    def1 = nextTable(&set.iterator);
    CuAssertTrue(tc, (def1 != NULL));
    /* test if the table is in fact the right one*/
#if DEBUG
    printf("testing names '%s' == '%s'\n",
           def1->metadata.name,
           tableName
    );
#endif
    CuAssertStrEquals(tc, def1->metadata.name, tableName);
#if DEBUG
    printf("testing hashes %X%X%X%X%X == %X%X%X%X%X\n",
           def1->tableId.a,
           def1->tableId.b,
           def1->tableId.c,
           def1->tableId.d,
           def1->tableId.e,
           test_hash1.a,
           test_hash1.b,
           test_hash1.c,
           test_hash1.d,
           test_hash1.e
    );
#endif
    CuAssertTrue(tc, equalsHashCode160(&def1->tableId, &test_hash1));

    /* test iterating past end */
#if DEBUG
    printf("iterate past end \n");
#endif
    def1 = nextTable(&set.iterator);
#if DEBUG
    printf("doAssert def \n");
#endif
    CuAssertTrue(tc, (def1 == NULL));
#if DEBUG
    printf("doAssert set.iterator \n");
#endif
    CuAssertTrue(tc, (set.iterator == NULL));
  }
  doneLocalStore(NULL);
}

void test_datalayerDataInsertionAndIteration(CuTest *tc)
{
  DHT_LocalStoreResultSet result;
  DHT_LocalStoreUnit * unit;
  HashCode160 *       hashResult;
  int ret;

#if DEBUG
  printf("test_datalayerDataInsertionAndIteration.\ninit store \n");
#endif
  initLocalStore(NULL);
#if DEBUG
  printf("create a table \n");
#endif
  if(localStoreCreateTable(&test_hash1, get_TableMeta("TAULU"), get_TableConfig()) == OK) {
    /* inserts data to table */
    unit = get_DataUnitFromStr("testdata");
#if DEBUG
    printf("store unit \n");
#endif
    ret = localStorePut(&test_hash1, unit);
    CuAssertIntEquals(tc, ret, OK);

#if DEBUG
    printf("store unit again \n");
#endif
    /* test inserting the same data again */
    unit = get_DataUnitFromStr("testdata");
    ret = localStorePut(&test_hash1, unit);
    CuAssertIntEquals(tc, ret, DHT_ERRORCODES__DUPLICATE_DATA_UNIT);

#if DEBUG
    printf("get iterator \n");
#endif
    /* result and iteration */
    result = localStoreGet(&test_hash1, &get_DataUnitFromStr("testdata")->hash);
    CuAssertIntEquals(tc, result.returnCode, OK);
#if DEBUG
    printf("iterate to next \n");
#endif
    unit  = nextUnit(&result.iterator);
    hashResult = &get_DataUnitFromStr("testdata")->hash;
#if DEBUG
    printf("testing hashes %X%X%X%X%X == %X%X%X%X%X\n",
           hashResult->a,
           hashResult->b,
           hashResult->c,
           hashResult->d,
           hashResult->e,
           unit->hash.a,
           unit->hash.b,
           unit->hash.c,
           unit->hash.d,
           unit->hash.e
    );
#endif
    CuAssertTrue(tc, equalsHashCode160(hashResult, &unit->hash));

    /* iteration over limit */
#if DEBUG
    printf("iterate past end \n");
#endif
    unit  = nextUnit(&result.iterator);
#if DEBUG
    printf("doAssert unit \n");
#endif
    CuAssertTrue(tc, unit == NULL);
#if DEBUG
    printf("doAssert iterator \n");
#endif
    CuAssertTrue(tc, result.iterator == NULL);
    
  }
  doneLocalStore(NULL);

}

void test_datalayerStoreUnitFlags(CuTest *tc)
{
#if DEBUG
  printf("testflags\n");
#endif
  DHT_LocalStoreUnit * unit = get_DataUnitFromStr("testdata");
  
  setMyData(unit, TRUE);
  setRepublishable(unit, TRUE);
#if DEBUG
  printf("flags: %d\n", unit->flags);
#endif
  CuAssertTrue(tc, isMyData(unit));
  CuAssertTrue(tc, isRepublishable(unit));

  setMyData(unit, FALSE);
#if DEBUG
  printf("flags: %d\n",unit->flags);
#endif
  CuAssertTrue(tc, !(isMyData(unit)));
  CuAssertTrue(tc, isRepublishable(unit));

  setRepublishable(unit, FALSE);
#if DEBUG
  printf("flags: %d\n",unit->flags);
#endif
  CuAssertTrue(tc, !(isRepublishable(unit)));
  CuAssertTrue(tc, !(isMyData(unit)));

  setMyData(unit, TRUE);
#if DEBUG
  printf("flags: %d\n",unit->flags);
#endif
  CuAssertTrue(tc, isMyData(unit));
  CuAssertTrue(tc, !(isRepublishable(unit)));
}

void test_datalayerDataInsertionAndDeletion(CuTest *tc)
{
  DHT_LocalStoreResultSet result;
  DHT_LocalStoreUnit * unit;
  DHT_DataContainer * test_data;
  HashCode160 *       hashResult;
  char * dataString;
  int ret;

#if DEBUG
  printf("test_datalayerDataInsertionAndDeletion.\ninit store \n");
#endif
  initLocalStore(NULL);
#if DEBUG
  printf("create a table \n");
#endif
  if(localStoreCreateTable(&test_hash1, get_TableMeta("TAULU"), get_TableConfig()) == OK) {
    /* inserts data to table */
#if DEBUG
    printf("store unit \n");
#endif
    unit = get_DataUnitFromStr("testdata");
    ret = localStorePut(&test_hash1, unit);
    CuAssertIntEquals(tc, ret, OK);

#if DEBUG
    printf("store another different unit \n");
#endif
    /* create another unit */
    hashResult        = &get_DataUnitFromStr("testdata")->hash;

    dataString = 
"testdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestda\
testdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestda\
testdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestda\
testdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestda\
testdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestda\
testdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestda\
testdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestda\
testdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestda\
testdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestdatatestda\
tatestdatatestdatatestdatatestdata";
    test_data         = MALLOC(sizeof(DHT_DataContainer));
    test_data->data   = dataString;
    test_data->dataLength = strlen(dataString);
    unit = new_DHT_LocalStoreUnit(hashResult, test_data);

    ret = localStorePut(&test_hash1, unit);
    CuAssertIntEquals(tc, ret, OK);

#if DEBUG
    printf("iterate all units \n");
#endif
    /* result and iteration */
    result = localStoreGet(&test_hash1, hashResult);
    CuAssertIntEquals(tc, result.returnCode, OK);
    ret = 0;
    while ((unit = nextUnit(&result.iterator)) != NULL) {
      CuAssertTrue(tc, equalsHashCode160(hashResult, &unit->hash));
      ret++;
    }
    CuAssertIntEquals(tc, ret, 2); /* both should've been found*/

#if DEBUG
    printf("iterate all units again\n");
#endif
    result = localStoreGet(&test_hash1, hashResult);
    CuAssertIntEquals(tc, result.returnCode, OK);
    ret = 0;
    while ((unit = nextUnit(&result.iterator)) != NULL) {
      CuAssertTrue(tc, equalsHashCode160(hashResult, &unit->hash));
      ret++;
    }
    CuAssertIntEquals(tc, ret, 2); /* both should've been found*/

#if DEBUG
    printf("iterate all units and delete them \n");
#endif
    /* iteration and deletion */
    result = localStoreGet(&test_hash1, hashResult);
    CuAssertIntEquals(tc, result.returnCode, OK);
    while ((unit = nextUnit(&result.iterator)) != NULL) {
#if DEBUG
      printf("delete ");
#endif
      CuAssertTrue(tc, equalsHashCode160(hashResult, &unit->hash));
      ret = localStoreDelete(&test_hash1, result.iterator);
#if DEBUG
    printf(" unit \n");
#endif
      CuAssertIntEquals(tc, ret, OK);
    }
#if DEBUG
    printf("iterate past end \n");
#endif
    unit  = nextUnit(&result.iterator);
    CuAssertTrue(tc, unit == NULL);
#if DEBUG
    printf("try get the units after deletion\n");
#endif
    result = localStoreGet(&test_hash1, hashResult);
    CuAssertIntEquals(tc, DHT_ERRORCODES__EMPTY_RESULT, result.returnCode);
    ret = 0;
    while ((unit = nextUnit(&result.iterator)) != NULL) {
      CuAssertTrue(tc, equalsHashCode160(hashResult, &unit->hash));
      ret++;
    }
    CuAssertTrue(tc, result.iterator == NULL);
    CuAssertIntEquals(tc, ret, 0); /* none should've been found*/
#if DEBUG
    printf("delete table \n");
#endif
    ret = localStoreDropTable(&test_hash1);
    CuAssertIntEquals(tc, ret, OK);
#if DEBUG
    printf("try to search the table \n");
#endif
    result = localStoreGet(&test_hash1, hashResult);
    CuAssertIntEquals(tc, DHT_ERRORCODES__TABLE_NOT_FOUND, result.returnCode);
    CuAssertTrue(tc, result.iterator == NULL);
  }
  doneLocalStore(NULL);

}



CuSuite* datalayer_GetSuite()
{
  printf("getSuite\n");
  CuSuite* suite = CuSuiteNew();
  SUITE_ADD_TEST(suite, test_datalayerStoreUnitFlags);
  SUITE_ADD_TEST(suite, test_datalayerTableCreation);
  SUITE_ADD_TEST(suite, test_datalayerDataInsertionAndIteration);
  SUITE_ADD_TEST(suite, test_datalayerDataInsertionAndDeletion);
  return suite;
}


#ifndef TEST_ALL
int main(void) {
  printf("start\n");
  CuString *output = CuStringNew();
  CuSuite* suite = CuSuiteNew();
        
  CuSuiteAddSuite(suite, datalayer_GetSuite());
  
  CuSuiteRun(suite);
  CuSuiteSummary(suite, output);
  CuSuiteDetails(suite, output);
  printf("%s\n", output->buffer);
  return 0; 
}
#endif
