 /*
      This file is part of GNUnet

      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
      by the Free Software Foundation; either version 2, or (at your
      option) any later version.

      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
      General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
      Free Software Foundation, Inc., 59 Temple Place - Suite 330,
      Boston, MA 02111-1307, USA.
 */

/**
 * @file module/test/dht_util_test.c 
 * @brief Unit tests for the functions of the DHT module
 * @author Risto Saarelma
 * @version $Id: dht_test.c,v 1.6 2004/05/18 19:01:07 rsaarelm Exp $
 */

#include <stdio.h>
#include <gnunet_util.h>
#include "../dht.h"
#include "../dht_util.h"
#include "CuTest.h"

static HashCode160 test_hash1 = {1,2,3,4, 5};
static CoreAPIForApplication hackz_coreAPI;

static int foobaarCS(const unsigned short type, CSHandler callback) {
/*  printf("foobaarCS %d \n",type);
  */return OK;
}
static int unfoobaarCS(ClientExitHandler callback)  {
/*  printf("unfoobaarCS\n");
  */return OK;
}
static int isfoobaar(const unsigned short type) {
/*  printf("isfoobaar %d \n",type);
  */return YES;
}

static int foobaarP2P(const unsigned short type, MessagePartHandler callback) {
/*  printf("foobaarP2P %d \n",type);
  */return OK;
}

static DHT_TableConfig * get_TableConfig(){
  DHT_TableConfig * res;

  res = MALLOC(sizeof(*res));
  res->replicationCount =0;
  res->parallelismCount=0;
  res->maximumValuesPerKey=1;
  res->expirationTimeSeconds=10;
  res->flags=0;
  res->cacheTimeMultiplier=0.0;
  return res;
}

static DHT_NodeAddress * get_NodeAddress(){
  DHT_NodeAddress * res;
  res = MALLOC(sizeof(*res));
  return res;
}


static DHT_TableMetaData * get_TableMeta(char * tableName){
  DHT_TableMetaData * res;

  res = MALLOC(sizeof(*res));
  res->name = tableName;

  return res;
}


void testDhtCreate(CuTest *tc)
{
  printf("doing testDhtCreate\n");
  HashCode160 * id; 
  int res;

  res = initialize_dht_protocol( &hackz_coreAPI );
  CuAssertIntEquals(tc, OK, res);
  
  res = create(get_TableConfig(), get_TableMeta("FooBar"),&id);
  CuAssertIntEquals(tc, OK, res);

  done_dht_protocol( &hackz_coreAPI );
  printf("done testDhtCreate\n");
}

void testDhtJoin(CuTest *tc)
{
  printf("doing testDhtJoin\n");
  int res;
  
  res = initialize_dht_protocol( &hackz_coreAPI );
  CuAssertIntEquals(tc, OK, res);

  res = join(get_NodeAddress(), &test_hash1);
  CuAssertIntEquals(tc, OK, res);

  done_dht_protocol( &hackz_coreAPI );
  printf("done testDhtJoin\n");
}

void testDhtLeave(CuTest *tc)
{
  printf("doing testDhtLeave\n");
  int res;
  
  res = initialize_dht_protocol( &hackz_coreAPI );
  CuAssertIntEquals(tc, OK, res);

  res = leave(&test_hash1);
  CuAssertIntEquals(tc, OK, res);

  done_dht_protocol( &hackz_coreAPI );
  printf("done testDhtLeave\n");
}

void testDhtInsert(CuTest *tc)
{
  printf("doing testDhtInsert\n");
  int res;
  
  char 
    *value = "FOOBAAR",
    *key = "MooZoo";

  res = initialize_dht_protocol( &hackz_coreAPI );
  CuAssertIntEquals(tc, OK, res);
  
  res = insert(&test_hash1, 
          newDataContainer(strlen(key)+1,key),
          newDataContainer(strlen(value)+1,value));
  CuAssertIntEquals(tc, OK, res);

  done_dht_protocol( &hackz_coreAPI );
  printf("done testDhtInsert\n");
}

void testDhtFetch(CuTest *tc)
{
  printf("doing testDhtFetch\n");
  int res;
  
  char *key = "MooZoo";
  DHT_ResultSet * result;

  res = initialize_dht_protocol( &hackz_coreAPI );
  CuAssertIntEquals(tc, OK, res);
  
  res = fetch(&test_hash1, newDataContainer(strlen(key)+1,key), &result);
  CuAssertIntEquals(tc, OK, res);

  done_dht_protocol( &hackz_coreAPI );
  printf("done testDhtFetch\n");
}

void testDhtTables(CuTest *tc)
{
  printf("doing testDhtTables\n");
  int res;
  HashCode160 * id; 
  DHT_TableSet * result;

  res = initialize_dht_protocol( &hackz_coreAPI );
  CuAssertIntEquals(tc, OK, res);

  res = create(get_TableConfig(), get_TableMeta("FooBar"),&id);
  CuAssertIntEquals(tc, OK, res);

  res = tables(get_NodeAddress(), &result);
  CuAssertIntEquals(tc, OK, res);

  done_dht_protocol( &hackz_coreAPI );
  printf("done testDhtTables\n");
}


void testDhtInserted(CuTest *tc)
{
  printf("doing testDhtInserted\n");
  int res;
  DHT_DataList * result;

  res = initialize_dht_protocol( &hackz_coreAPI );
  CuAssertIntEquals(tc, OK, res);

  res = inserted(&test_hash1, &result);
  CuAssertIntEquals(tc, OK, res);

  done_dht_protocol( &hackz_coreAPI );
  printf("done testDhtInserted\n");
}

void testDhtDrop(CuTest *tc)
{
  printf("doing testDhtDrop\n");
  int res;

  res = initialize_dht_protocol( &hackz_coreAPI );
  CuAssertIntEquals(tc, OK, res);

  DHT_StoredDataReference storedDataRef;
  res = drop(&test_hash1, &storedDataRef);
  CuAssertIntEquals(tc, OK, res);

  done_dht_protocol( &hackz_coreAPI );
  printf("done testDhtDrop\n");
}

CuSuite * getDhtTestSuite() {
  CuSuite * suite = CuSuiteNew();

  /* TODO: Some tests */
  SUITE_ADD_TEST(suite, testDhtCreate);
  SUITE_ADD_TEST(suite, testDhtJoin);
  SUITE_ADD_TEST(suite, testDhtLeave);
  SUITE_ADD_TEST(suite, testDhtInsert);
  SUITE_ADD_TEST(suite, testDhtFetch);
  SUITE_ADD_TEST(suite, testDhtTables);
  SUITE_ADD_TEST(suite, testDhtInserted);
  SUITE_ADD_TEST(suite, testDhtDrop);
  return suite;
}

int main(int argc, char * argv[]) 
{

  printf("start testing dht\n");
  hackz_coreAPI.myIdentity = get_NodeAddress();
  hackz_coreAPI.registerHandler = &foobaarP2P;
  hackz_coreAPI.isHandlerRegistered = &isfoobaar;
  hackz_coreAPI.unregisterHandler = &foobaarP2P;

  hackz_coreAPI.registerClientHandler = &foobaarCS;
  hackz_coreAPI.unregisterClientHandler = &foobaarCS;
  hackz_coreAPI.registerClientExitHandler = &unfoobaarCS;
  hackz_coreAPI.unregisterClientExitHandler = &unfoobaarCS;
  hackz_coreAPI.isClientHandlerRegistered = &isfoobaar;

  CuString * output = CuStringNew();
  CuSuite * suite = getDhtTestSuite();

  CuSuiteRun(suite);

  CuSuiteSummary(suite, output);
  CuSuiteDetails(suite, output);
  printf("# Results of dht_test:\n");
  printf("%s\n", output->buffer);

  return suite->failCount;
}
