 /*
      This file is part of GNUnet

      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
      by the Free Software Foundation; either version 2, or (at your
      option) any later version.

      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
      General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
      Free Software Foundation, Inc., 59 Temple Place - Suite 330,
      Boston, MA 02111-1307, USA.
 */

/**
 * @file module/test/dht_util_test.c
 * @brief Unit tests for the common utility functions of the DHT module
 * @author Risto Saarelma
 * @version $Id: dht_util_test.c,v 1.5 2004/05/06 10:22:27 szviitan Exp $
 */

#include <stdio.h>
#include "dht_util.h"
#include "CuTest.h"

void testDhtUtilConversion(CuTest *tc)
{
  DHT_DataContainer key1,value1,* data,* key2,* value2;
  HashCode160 * keyHash, checkHash;
  char * str1, * str2;
  int res;
  
  printf("doing conversion test\n");
  
  str1 = "testdatatestdatatestdatatestdatatestdatatestdatatestdatatestkey1";
  key1.data = str1;
  key1.dataLength = strlen(str1)+1; /* include also the null char*/
  hash(key1.data, key1.dataLength, &checkHash);

  str2 = "testdatatestdatatestdatatestdatatestdatatestdatatestdatatestvalue1";
  value1.data = str2;
  value1.dataLength = strlen(str2)+1; /* include also the null char*/

  /* convert to */
  res = convertKeyValueToHashNData(&key1,&value1,&keyHash, &data);
  CuAssertIntEquals(tc, res,OK);
  CuAssertTrue(tc, (keyHash != NULL));
  printf("%s ==", hash160ToString(keyHash));
  printf(" %s\n", hash160ToString(&checkHash));
  CuAssertTrue(tc, equalsHashCode160(keyHash, &checkHash));
  CuAssertTrue(tc, (data != NULL));
  CuAssertTrue(tc, (data != NULL));
  res = key1.dataLength + value1.dataLength + 2*sizeof(DHT_DataContainer);
  CuAssertIntEquals(tc, res, data->dataLength);

  checkHash.a = 0;
  printf("%s ==", hash160ToString(keyHash));
  printf(" %s\n", hash160ToString(&checkHash));

  /* convert back */
  res = convertDataToKeyNValue(data, &key2, &value2);
  CuAssertIntEquals(tc, OK, res);
  CuAssertTrue(tc, (key2 != NULL));
  CuAssertTrue(tc, (value2 != NULL));
  CuAssertStrEquals(tc, str1, (char *)key2->data);
  CuAssertStrEquals(tc, str2, (char *)value2->data);
  /* test freeing */
  printf("free value\n");
  freeDataContainer(value2);
  printf("free key\n");
  freeDataContainer(key2);  
}

void testDhtDataContainerEquality(CuTest *tc)
{
  DHT_DataContainer data1,data2,data3;
  char * str1, * str2, * str3;
  
  printf("doing equality test\n");
  
  str1 = "testdatatestdatatestdatatestdatatestdatatestdatatestdatatest";
  data1.data = str1;
  data1.dataLength = strlen(str1)+1; /* include also the null char*/

  str2 = "testdatatestdatatestdatatestdatatestdatatestdatatestdatatest";
  data2.data = str1;
  data2.dataLength = strlen(str1)+1; /* include also the null char*/

  str3 = "foobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobar"
         "foobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobar";
  data3.data = str3;
  data3.dataLength = strlen(str3)+1; /* include also the null char*/

  /* convert to */
  CuAssertTrue(tc, equalsDataContainers(&data1, &data2));
  CuAssertTrue(tc, !equalsDataContainers(&data1,&data3));
  data3.dataLength = data2.dataLength;
  CuAssertTrue(tc, !equalsDataContainers(&data2,&data3));
  
}

CuSuite * getDhtUtilTestSuite() {
  CuSuite * suite = CuSuiteNew();

  /* TODO: Some tests */
  SUITE_ADD_TEST(suite, testDhtUtilConversion);
  SUITE_ADD_TEST(suite, testDhtDataContainerEquality);
  return suite;
}

int main(int argc, char * argv[]) {
  CuString * output = CuStringNew();
  CuSuite * suite = getDhtUtilTestSuite();

  CuSuiteRun(suite);

  CuSuiteSummary(suite, output);
  CuSuiteDetails(suite, output);
  printf("# Results of dht_util_test:\n");
  printf("%s\n", output->buffer);

  return suite->failCount;
}
