 /*
      This file is part of GNUnet

      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
      by the Free Software Foundation; either version 2, or (at your
      option) any later version.

      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
      General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
      Free Software Foundation, Inc., 59 Temple Place - Suite 330,
      Boston, MA 02111-1307, USA.
 */

/**
 * @file module/test/kademlia_test.c
 * @brief Unit tests for the internal structures of the Kademlia DHT system
 * @author Risto Saarelma
 * @version $Id: kademlia_test.c,v 1.3 2004/05/05 14:45:42 rsaarelm Exp $
 */
#include <stdio.h>
#include "kademlia.h"
#include "dht_util.h"
#include "rpc.h"
#include "CuTest.h"

void randomHashCode(HashCode160 * code) {
  /* tryhex2hashOrHashString returns random code if 1st param is null.
   */
  tryhex2hashOrHashString(NULL, code);
}

/**
 * Fill hash with gibberish determined by string provided.
 */
void genHash(const char * data, HashCode160 * target) {
  tryhex2hashOrHashString(data, target);
}

void tableCreationTest(CuTest * tc) {
  DHT_TableId tableId;
  DHT_NodeAddress localId;

  DHT_TableId otherTable;
  DHT_NodeAddress otherNode;

  Kademlia_RoutingTable * table;
  Kademlia_RoutingTable * table2;
  int bucketSize = 30;

  genHash("xyzzy", &tableId);
  genHash("foobar", &localId.hashPubKey);

  genHash("klmml", &otherTable);
  genHash("sbbone", &otherNode.hashPubKey);

  printf("Created tableId:    %s\n", hash160ToString(&tableId));
  printf("Created otherTable: %s\n", hash160ToString(&otherTable));

  Kademlia_clearTables();

  table2 = Kademlia_tableLookup(&tableId);
  CuAssertPtrEquals(tc, NULL, table2);

  table = Kademlia_registerTable(
      &tableId,
      &localId,
      bucketSize);

  /*
  Kademlia_registerRpcCalls();

  CuAssert(
      tc,
      "Node does not respond to DHT ping self", 
      dhtPingNode(&tableId, &localId));

   */

  
}

CuSuite * getKademliaTestSuite() {
  CuSuite * suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, tableCreationTest);

  return suite;
}

int main(int argc, char * argv[]) {
  CuString * output = CuStringNew();
  CuSuite * suite = getKademliaTestSuite();

  CuSuiteRun(suite);

  CuSuiteSummary(suite, output);
  CuSuiteDetails(suite, output);
  printf("# Results of kademlia_test:\n");
  printf("%s\n", output->buffer);

  return suite->failCount;
}
