
/* [BUG] Insert the GNU copyright here! */

/* dht-create.c */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <gnunet_util.h>
#include "../include/dht_api.h"

static void printHelp()
{
  printf("USAGE: dht-create [OPTIONS] "
	 "TABLE\n");
  printHelpDefault();
}

static int parseOptions(int argc,
			char ** argv)
{
  int c;  

  while (1) {
    int option_index = 0;
    static struct GNoption long_options[] = {
      LONG_DEFAULT_OPTIONS,
      { 0,0,0,0 }
    };
    c = GNgetopt_long(argc,
		      argv,
		      "vh" "H:c:L:d",
		      long_options, 
		      &option_index);    
    if (c == -1) 
      break;  /* No more flags to process */
    if (YES == parseDefaultOptions(c, GNoptarg))
      continue;
    switch(c) {
    case 'v': 
      printf("dht-create: %s\n", "$Id: dht-create.c,v 1.5 2004/05/20 11:32:33 tmtoivon Exp $");
      return SYSERR;
    case 'h': 
      printHelp(); 
      return SYSERR;
    default:
      LOG(LOG_FAILURE,
	  "Unknown option %c. Aborting.\n"
	  "Use --help to get a list of options.\n",
	  c);
      return SYSERR;
    } /* end of parsing commandline */
  } /* while (1) */
  if (argc - GNoptind != 1) {
    LOG(LOG_WARNING, 
	"Not enough arguments. "
	"You must specify at least a table\n");
    printHelp();
    return SYSERR;
  }
  FREENONNULL(setConfigurationString("DHT-CREATE",
				     "TABLE",
				     argv[GNoptind++]));
  return OK;
}

int main(int argc, char **argv)
{
  int errorCode; 
  DHT_TableHandle *handle;
  DHT_TableConfig config;
  DHT_TableMetaData metadata;
  HexName hex;

  /* -- parse command line parameters -- */

  if (SYSERR == initUtil(argc, argv, &parseOptions)) {
    return 0; /* parse error, --help, etc. */
  }

  if (argc != 2) {
    fprintf(stderr, "USAGE: %s <name>\n", argv[0]);
    return EXIT_FAILURE;
  }

  /* -- create table -- */

  if (isError(errorCode = initializeApi())) {
    return errorCode;
  }

  config.replicationCount = 1;
  config.parallelismCount = 1;
  config.maximumValuesPerKey = 1;
  config.expirationTimeSeconds = 1800;
  config.flags = 0;
  config.cacheTimeMultiplier = 1.0;

  metadata.name = argv[1];

  handle = create(&metadata, &config);

  LOG(LOG_DEBUG, "create(), error code: %d\n", handle->errorCode);
  
  if (isError(handle->errorCode)) {
    LOG(LOG_WARNING, "create() returned an error: %d\n",
	handle->errorCode);
  }

  hash2hex(&(handle->tableId), &hex);
  printf("%s\n", hex.data);

  destroyApi(); 

  if (isError(handle->errorCode)) {
    return handle->errorCode;
  }
  
  return EXIT_SUCCESS; 
}
