
/* [BUG] Insert the GNU copyright here! */

/* dht-fetch.c - Fetches a key-value pair based on a table identifier
   and a key. */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <gnunet_util.h>
#include "../include/dht_api.h"

static void printHelp()
{
  printf("USAGE: dht-fetch [OPTIONS] "
	 "TABLE KEY\n");
  printHelpDefault();
}

static int parseOptions(int argc,
			char ** argv)
{
  int c;  

  while (1) {
    int option_index = 0;
    static struct GNoption long_options[] = {
      LONG_DEFAULT_OPTIONS,
      { 0,0,0,0 }
    };
    c = GNgetopt_long(argc,
		      argv,
		      "vh" "H:c:L:d",
		      long_options, 
		      &option_index);    
    if (c == -1) 
      break;  /* No more flags to process */
    if (YES == parseDefaultOptions(c, GNoptarg))
      continue;
    switch(c) {
    case 'v': 
      printf("dht-fetch: %s\n", "$Id: dht-fetch.c,v 1.6 2004/05/24 10:18:12 mjraiha Exp $");
      return SYSERR;
    case 'h': 
      printHelp(); 
      return SYSERR;
    default:
      LOG(LOG_FAILURE,
	  "Unknown option %c. Aborting.\n"
	  "Use --help to get a list of options.\n",
	  c);
      return SYSERR;
    } /* end of parsing commandline */
  } /* while (1) */
  if (argc - GNoptind != 2) {
    LOG(LOG_WARNING, 
	"Not enough arguments. "
	"You must specify at least a node and a table\n");
    printHelp();
    return SYSERR;
  }
  FREENONNULL(setConfigurationString("DHT-FETCH",
				     "TABLE",
				     argv[GNoptind++]));
  FREENONNULL(setConfigurationString("DHT-FETCH",
				     "KEY",
				     argv[GNoptind++]));
  return OK;
}

int main(int argc, char **argv)
{
  int errorCode; 

  DHT_TableHandle handle;
  DHT_DataContainer key;
  DHT_ResultSet *set;
  DHT_DataContainer *item;
  HexName hex;

  /* -- parse command line parameters -- */

  if (SYSERR == initUtil(argc, argv, &parseOptions)) {
    return EXIT_FAILURE; /* parse error, --help, etc. */
  }

  /* -- fetch -- */

  /* HT_TableHandle * table */

  if (strlen(getConfigurationString("DHT-FETCH", "TABLE")) != 40) {
    fprintf(stderr, "%s: 'table' is not forty hex digits\n", argv[0]);
    return EXIT_FAILURE;
  }

  strncpy(hex.data, getConfigurationString("DHT-FETCH", "TABLE"), 41);
  hex2hash(&hex, &(handle.tableId));

  /* DHT_DataContainer * key */
  
  key.dataLength = strlen(getConfigurationString("DHT-FETCH", "KEY")) + 1;
  key.data = getConfigurationString("DHT-FETCH", "KEY");

  /* do stuff */

  if (isError(errorCode = initializeApi())) {
    return errorCode;
  }

  set = fetch(&handle, &key);
  
  LOG(LOG_DEBUG, "fetch, error code: %d\n", set->errorCode);
  
  if (isError(set->errorCode)) {
    LOG(LOG_WARNING, "fetch() returned an error: %d\n",
	set->errorCode);
  }

  while (resultSetNext(set, &item) == OK) {
    if (item == NULL) {
      LOG(LOG_WARNING, "oops, item is NULL\n");
    }
    /* [BUG] We just assume here that data is a null-terminated string. */
    printf("%s\n", (char *) item->data);
  }

  destroyApi();

  /* [BUG] Is errorCode still valid here? */

  LOG(LOG_DEBUG, "set->errorCode: %d\n", set->errorCode);

  if (isError(set->errorCode)) {
    return set->errorCode;
  }

  return EXIT_SUCCESS; 
}
