
/* [BUG] Insert the GNU copyright here! */

/* dht-insert.c */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <gnunet_util.h>
#include "../include/dht_api.h"

static void printHelp()
{
  printf("USAGE: dht-insert [OPTIONS] "
	 "TABLE KEY VALUE\n");
  printHelpDefault();
}

static int parseOptions(int argc,
			char ** argv)
{
  int c;  

  while (1) {
    int option_index = 0;
    static struct GNoption long_options[] = {
      LONG_DEFAULT_OPTIONS,
      { 0,0,0,0 }
    };
    c = GNgetopt_long(argc,
		      argv,
		      "vh" "H:c:L:d",
		      long_options, 
		      &option_index);    
    if (c == -1) 
      break;  /* No more flags to process */
    if (YES == parseDefaultOptions(c, GNoptarg))
      continue;
    switch(c) {
    case 'v': 
      printf("dht-insert: %s\n", "$Id: dht-insert.c,v 1.5 2004/05/20 11:32:33 tmtoivon Exp $");
      return SYSERR;
    case 'h': 
      printHelp(); 
      return SYSERR;
    default:
      LOG(LOG_FAILURE,
	  "Unknown option %c. Aborting.\n"
	  "Use --help to get a list of options.\n",
	  c);
      return SYSERR;
    } /* end of parsing commandline */
  } /* while (1) */
  if (argc - GNoptind != 3) {
    LOG(LOG_WARNING, 
	"Not enough arguments. "
	"You must specify at least a table, a key and a value\n");
    printHelp();
    return SYSERR;
  }
  FREENONNULL(setConfigurationString("DHT-INSERT",
				     "TABLE",
				     argv[GNoptind++]));
  FREENONNULL(setConfigurationString("DHT-INSERT",
				     "KEY",
				     argv[GNoptind++]));
  FREENONNULL(setConfigurationString("DHT-INSERT",
				     "VALUE",
				     argv[GNoptind++]));
  return OK;
}

int main(int argc, char **argv)
{
  int errorCode; 

  DHT_TableHandle handle;
  DHT_DataContainer key, value;
  HexName hex;

  /* -- parse command line parameters -- */

  if (SYSERR == initUtil(argc, argv, &parseOptions)) {
    return EXIT_FAILURE; /* parse error, --help, etc. */
  }

  /* -- insert -- */

  /* DHT_TableHandle * table */

  if (strlen(getConfigurationString("DHT-INSERT", "TABLE")) != 40) {
    fprintf(stderr, "%s: 'node' is not forty hex digits\n", argv[0]);
    return EXIT_FAILURE;
  }

  strncpy(hex.data, getConfigurationString("DHT-INSERT", "TABLE"), 41);
  hex2hash(&hex, &(handle.tableId));

  /* DHT_DataContainer * key */
  
  /* [NOTE] We use a NULL terminated string; maybe good, maybe not. */

  key.dataLength = strlen(getConfigurationString("DHT-INSERT", "KEY"))+1;
  key.data = getConfigurationString("DHT-INSERT", "KEY");

  /* DHT_DataContainer * value */

  value.dataLength = strlen(getConfigurationString("DHT-INSERT", "VALUE"))+1;
  value.data = getConfigurationString("DHT-INSERT", "VALUE");

  /* do stuff */

  if (isError(errorCode = initializeApi())) {
    return errorCode;
  }

  errorCode = insert(&handle, &key, &value);
  
  destroyApi(); 

  LOG(LOG_DEBUG, "errorCode: %d\n", errorCode);


  if (isError(errorCode)) {
    return errorCode;
  }

  return EXIT_SUCCESS; 
}
