
/* [BUG] Insert the GNU copyright here! */

/* dht-join.c - Joins an existing DHT table. A table identifier (forty
   hex digits) will output followed by a newline. Return status is
   success (0) if join succeeded and the table identifier is valid;
   otherwise return status indicates the failure condition and
   identifier is not to be used.*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <gnunet_util.h>
#include "../include/dht_api.h"

static void printHelp()
{
  printf("USAGE: dht-join [OPTIONS] "
	 "NODE TABLE\n");
  printHelpDefault();
}

static int parseOptions(int argc,
			char ** argv)
{
  int c;  

  while (1) {
    int option_index = 0;
    static struct GNoption long_options[] = {
      LONG_DEFAULT_OPTIONS,
      { 0,0,0,0 }
    };
    c = GNgetopt_long(argc,
		      argv,
		      "vh" "H:c:L:d",
		      long_options, 
		      &option_index);    
    if (c == -1) 
      break;  /* No more flags to process */
    if (YES == parseDefaultOptions(c, GNoptarg))
      continue;
    switch(c) {
    case 'v': 
      printf("dht-join: %s\n", "$Id: dht-join.c,v 1.11 2004/05/16 03:27:21 tmtoivon Exp $");
      return SYSERR;
    case 'h': 
      printHelp(); 
      return SYSERR;
    default:
      LOG(LOG_FAILURE,
	  "Unknown option %c. Aborting.\n"
	  "Use --help to get a list of options.\n",
	  c);
      return SYSERR;
    } /* end of parsing commandline */
  } /* while (1) */
  if (argc - GNoptind != 2) {
    LOG(LOG_WARNING, 
	"Not enough arguments. "
	"You must specify atleast a node and a table\n");
    printHelp();
    return SYSERR;
  }
  FREENONNULL(setConfigurationString("DHT-JOIN",
				     "NODE",
				     argv[GNoptind++]));
  FREENONNULL(setConfigurationString("DHT-JOIN",
				     "TABLE",
				     argv[GNoptind++]));
  return OK;
}

int main(int argc, char **argv)
{
  int errorCode; 

  DHT_NodeAddress node; 
  DHT_TableId table; 
  DHT_TableHandle *handle;
  HexName hex;

  /* -- parse command line parameters -- */

  if (SYSERR == initUtil(argc, argv, &parseOptions)) {
    return EXIT_FAILURE; /* parse error, --help, etc. */
  }

  /* -- join table -- */

  if (strlen(getConfigurationString("DHT-JOIN", "NODE")) != 40) {
    fprintf(stderr, "%s: 'node' is not forty hex digits\n", argv[0]);
    return EXIT_FAILURE;
  }

  strncpy(hex.data, getConfigurationString("DHT-JOIN", "NODE"), 41);
  hex2hash(&hex, &(node.hashPubKey));

  if (strlen(getConfigurationString("DHT-JOIN", "TABLE")) != 40) {
    fprintf(stderr, "%s: 'table' is not forty hex digits\n", argv[0]);
    return EXIT_FAILURE;
  }

  strncpy(hex.data, getConfigurationString("DHT-JOIN", "TABLE"), 41);
  hex2hash(&hex, &table);

  if (isError(errorCode = initializeApi())) {
    return errorCode;
  }
  
  handle = join(&node, &table);

  if (isError(handle->errorCode)) {
    LOG(LOG_WARNING, "list() returned an error: %d\n",
	handle->errorCode);
  }

  destroyApi(); 

  LOG(LOG_DEBUG, "handle->errorCode: %d\n", handle->errorCode);

  hash2hex(&(handle->tableId), &hex);

  printf("%s\n", hex.data);

  if (isError(handle->errorCode)) {
    return handle->errorCode;
  }

  return EXIT_SUCCESS; 
}
