
/* [BUG] Insert the GNU copyright here! */

/* dht-list.c - Lists tables that a node has joined to. As input
   parameters give node identifier (forty hex digits). Output will
   consist of one line per joined table. Each line will have first the
   forty hex digit identifier of the table followed by whitespace and
   then the name of the table. */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <gnunet_util.h>
#include "../include/dht_api.h"

static void printHelp()
{
  printf("USAGE: dht-list [OPTIONS] "
	 "NODE\n");
  printHelpDefault();
}

static int parseOptions(int argc,
			char ** argv)
{
  int c;  

  while (1) {
    int option_index = 0;
    static struct GNoption long_options[] = {
      LONG_DEFAULT_OPTIONS,
      { 0,0,0,0 }
    };
    c = GNgetopt_long(argc,
		      argv,
		      "vh" "H:c:L:d",
		      long_options, 
		      &option_index);    
    if (c == -1) 
      break;  /* No more flags to process */
    if (YES == parseDefaultOptions(c, GNoptarg))
      continue;
    switch(c) {
    case 'v': 
      printf("dht-list: %s\n", "$Id: dht-list.c,v 1.9 2004/05/20 11:32:33 tmtoivon Exp $");
      return SYSERR;
    case 'h': 
      printHelp(); 
      return SYSERR;
    default:
      LOG(LOG_FAILURE,
	  "Unknown option %c. Aborting.\n"
	  "Use --help to get a list of options.\n",
	  c);
      return SYSERR;
    } /* end of parsing commandline */
  } /* while (1) */
  if (argc - GNoptind != 1) {
    LOG(LOG_WARNING, 
	"Not enough arguments. "
	"You must specify at least a node\n");
    printHelp();
    return SYSERR;
  }
  FREENONNULL(setConfigurationString("DHT-LIST",
				     "NODE",
				     argv[GNoptind++]));
  return OK;
}

int main(int argc, char **argv)
{
  int errorCode; 

  DHT_NodeAddress node;
  DHT_TableSet *set;
  DHT_TableSetItem *item;

  HexName hex;

  /* -- parse command line parameters -- */

  if (SYSERR == initUtil(argc, argv, &parseOptions)) {
    return EXIT_FAILURE; /* parse error, --help, etc. */
  }

  /* -- list tables -- */
  
  if (isError(errorCode = initializeApi())) {
    return errorCode;
  }

  if (strlen(getConfigurationString("DHT-LIST", "NODE")) != 40) {
    fprintf(stderr, "%s: 'node' is not forty hex digits\n", argv[0]);
    return EXIT_FAILURE;
  }

  strncpy(hex.data, getConfigurationString("DHT-LIST", "NODE"), 41);
  hex2hash(&hex, &(node.hashPubKey));

  set = listTables(&node);

  LOG(LOG_DEBUG, "listTables(), error code: %d\n", set->errorCode);
  
  if (isError(set->errorCode)) {
    LOG(LOG_WARNING, "listTables() returned an error: %d\n",
	set->errorCode);
  }

  while (tableSetNext(set, &item) == OK) {
    if (item == NULL) {
      LOG(LOG_WARNING, "oops, item is NULL\n");
      return EXIT_FAILURE;
    }
    hash2hex(&(item->tableId), &hex);
    printf("%s %s\n", hex.data, (item->tableMetaData).name);
  }

  destroyApi(); 

  if (isError(set->errorCode)) {
    return set->errorCode;
  }

  return EXIT_SUCCESS; 
}
