/*
      This file is part of GNUnet

      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
      by the Free Software Foundation; either version 2, or (at your
      option) any later version.

      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
      General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
      Free Software Foundation, Inc., 59 Temple Place - Suite 330,
      Boston, MA 02111-1307, USA.
 */

/**
 * @file util/networking.c
 * @brief Utility function implementations for DHT's networking section. 
 * @author Tomi Tukiainen
 * @version $Id: networking.c,v 1.1 2004/04/29 18:54:40 ttukiain Exp $ 
 */

#include <dht_api.h>

/**
 * Copies hashcode pointed by src to address pointed by dst 
 * and transforms dst's components a,b,c,d,e to Host Byte Order. 
 * 
 * @param dst destination HashCode
 * @param src source HashCode
 * @return pointer to dst
 */
HashCode160 * ntohHashCode160Copy(HashCode160 * dst, HashCode160 * src) {
  dst->a = ntohl(src->a);
  dst->b = ntohl(src->b);
  dst->c = ntohl(src->c);
  dst->d = ntohl(src->d);
  dst->e = ntohl(src->e);
  return dst; 
}

/**
 * Copies hashcode pointed by src to address pointed by dst 
 * and transforms dst's components a,b,c,d,e to Network Byte Order. 
 * 
 * @param dst destination HashCode
 * @param src source HashCode
 * @return pointer to dst
 */
HashCode160 * htonHashCode160Copy(HashCode160 * dst, HashCode160 * src) {
  dst->a = htonl(src->a);
  dst->b = htonl(src->b);
  dst->c = htonl(src->c);
  dst->d = htonl(src->d);
  dst->e = htonl(src->e);
  return dst; 
}



