/*
     This file is part of GNUnet.
     (C) 2001, 2002 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @file applications/template/template.c
 **/

#include "util/semaphore.h"
#include "util/hashing.h"
#include "util/logging.h"

static CoreAPIForApplication * coreAPI = NULL;
static GNUNET_TCP_SOCKET * client;
static Mutex lock;

static int handlep2pMSG(HostIdentity * sender,
		        p2p_HEADER * message) {
  return OK;
}

static void csHandle(GNUNET_TCP_SOCKET * client,
		     CS_HEADER * message) {
}

static void clientExitHandler(GNUNET_TCP_SOCKET * c) {
  MUTEX_LOCK(&lock);
  if (c == client)
    client = NULL;
  MUTEX_UNLOCK(&lock);
}

int initialize_template_protocol(CoreAPIForApplication * capi) {
  int ok = OK;

  MUTEX_CREATE(&lock);
  client = NULL;
  coreAPI = capi;

  if (SYSERR == capi->registerHandler(TEMPLATE_p2p_PROTO_TEST,
				      &handlep2pMSG))
    ok = SYSERR;
  if (SYSERR == capi->registerClientExitHandler(&clientExitHandler))
    ok = SYSERR;
  if (SYSERR == capi->registerClientHandler(TEMPLATE_CS_PROTO_TEST,
					    (CSHandler)&csHandle))
    ok = SYSERR;
  return ok;
}

void done_template_protocol() {
  capi->unregisterHandler(TEMPLATE_p2p_PROTO_TEST,
			   &handlep2pMSG);
  capi->unregisterClientExitHandler(&clientExitHandler);
  capi->unregisterClientHandler(TEMPLATE_CS_PROTO_TEST,
				(CSHandler)&csHandle);
  MUTEX_DESTROY(&lock);
  coreAPI = NULL;
}

/* end of template.c */
